@@ -1,4 +1,4 @@
 # Blocks a string key value from being interpreted for case conversion
-class OpaqueKey(str):
+class OpaqueKey(str):  # noqa: FURB189
     def __new__(cls, key: str) -> "OpaqueKey":
         return str.__new__(cls, key)
