@@ -13,14 +13,25 @@ from enum import Enum, StrEnum, auto
 from typing import Any, Self
 
 from . import util
-from .cross_output_links import CrossOutputPaths
+from .builder_types import CrossOutputPaths
 from .non_discriminated_union_exceptions import NON_DISCRIMINATED_UNION_EXCEPTIONS
-from .util import parse_type_str, unused
+from .util import parse_type_str
 
 RawDict = dict[Any, Any]
 EndpointKey = str
 
 
+class PathMapping(StrEnum):
+    NO_MAPPING = "no_mapping"
+    DEFAULT_MAPPING = "default_mapping"
+
+
+@dataclass(kw_only=True)
+class APIEndpointInfo:
+    root_path: str
+    path_mapping: PathMapping
+
+
 class StabilityLevel(StrEnum):
     """These are currently used for open api,
     see: https://github.com/Tufin/oasdiff/blob/main/docs/STABILITY.md
@@ -297,6 +308,7 @@ class SpecTypeDefn(SpecType):
         self._is_value_to_string = False
         self._is_valid_parameter = True
         self._is_dynamic_allowed = False
+        self._default_extant: PropertyExtant | None = None
         self.ext_info: Any = None
 
     def is_value_converted(self) -> bool:
@@ -329,6 +341,7 @@ class SpecTypeDefn(SpecType):
                 "ext_info",
                 "label",
                 "is_dynamic_allowed",
+                "default_extant",
             ]
             + extra_names,
         )
@@ -340,6 +353,10 @@ class SpecTypeDefn(SpecType):
         assert isinstance(is_dynamic_allowed, bool)
         self._is_dynamic_allowed = is_dynamic_allowed
 
+        default_extant = data.get("default_extant")
+        if default_extant is not None:
+            self._default_extant = PropertyExtant(default_extant)
+
     def _process_property(
         self, builder: SpecBuilder, spec_name: str, data: RawDict
     ) -> SpecProperty:
@@ -358,18 +375,18 @@ class SpecTypeDefn(SpecType):
             ],
         )
         try:
-            extant_type = data.get("extant")
+            extant_type_str = data.get("extant")
+            extant_type = (
+                PropertyExtant(extant_type_str) if extant_type_str is not None else None
+            )
+            extant = extant_type or self._default_extant
             if spec_name.endswith("?"):
-                if extant_type is not None:
+                if extant is not None:
                     raise Exception("cannot specify extant with ?")
                 extant = PropertyExtant.optional
                 name = spec_name[:-1]
             else:
-                extant = (
-                    PropertyExtant.required
-                    if extant_type is None
-                    else PropertyExtant(extant_type)
-                )
+                extant = extant or PropertyExtant.required
                 name = spec_name
 
             property_name_case = self.name_case
@@ -734,6 +751,8 @@ class SpecTypeDefnStringEnum(SpecTypeDefn):
                     builder.ensure(
                         isinstance(enum_value, str), "enum value should be string"
                     )
+                    assert isinstance(enum_value, str)
+
                     deprecated = value.get("deprecated", False)
                     builder.ensure(
                         isinstance(deprecated, bool),
@@ -835,7 +854,7 @@ class _EndpointPathDetails:
 
 
 def _resolve_endpoint_path(
-    path: str, api_endpoints: dict[EndpointKey, str]
+    path: str, api_endpoints: dict[EndpointKey, APIEndpointInfo]
 ) -> _EndpointPathDetails:
     root_path_source = path.split("/")[0]
     root_match = RE_ENDPOINT_ROOT.fullmatch(root_path_source)
@@ -843,7 +862,7 @@ def _resolve_endpoint_path(
         raise Exception(f"invalid-api-path-root:{root_path_source}")
 
     root_var = root_match.group(1)
-    root_path = api_endpoints[root_var]
+    root_path = api_endpoints[root_var].root_path
 
     _, *rest_path = path.split("/", 1)
     resolved_path = "/".join([root_path] + rest_path)
@@ -911,6 +930,7 @@ class SpecEndpoint:
     stability_level: StabilityLevel | None
     # Don't emit TypeScript endpoint code
     suppress_ts: bool
+    deprecated: bool = False
     async_batch_path: str | None = None
     result_type: ResultType = ResultType.json
     has_attachment: bool = False
@@ -928,13 +948,13 @@ class SpecEndpoint:
         pass
 
     def process(self, builder: SpecBuilder, data: RawDict) -> None:
-        unused(builder)
         util.check_fields(
             data,
             [
                 "method",
                 "path",
                 "data_loader",
+                "deprecated",
                 "is_sdk",
                 "stability_level",
                 "async_batch_path",
@@ -954,6 +974,7 @@ class SpecEndpoint:
         data_loader = data.get("data_loader", False)
         assert isinstance(data_loader, bool)
         self.data_loader = data_loader
+        self.deprecated = data.get("deprecated", False)
 
         is_sdk = data.get("is_sdk", EndpointEmitType.EMIT_NOTHING)
 
@@ -1080,7 +1101,7 @@ def _parse_const(
         elif const_type.defn_type.name == BaseTypeName.s_dict:
             assert isinstance(value, dict)
             builder.ensure(
-                len(const_type.parameters) == 2, "constant-dict-expects-one-type"
+                len(const_type.parameters) == 2, "constant-dict-expects-two-types"
             )
             key_type = const_type.parameters[0]
             value_type = const_type.parameters[1]
@@ -1129,6 +1150,11 @@ def _parse_const(
             )
             return value
 
+        if not const_type.is_base:
+            # IMPROVE: validate the object type properties before emission stage
+            builder.ensure(isinstance(value, dict), "invalid value for object constant")
+            return value
+
     raise Exception("unsupported-const-scalar-type", const_type)
 
 
@@ -1250,7 +1276,8 @@ class SpecNamespace:
 
             assert util.is_valid_type_name(name), f"{name} is not a valid type name"
             assert name not in self.types, f"{name} is duplicate"
-            defn_type = defn["type"]
+            defn_type = defn.get("type")
+            assert isinstance(defn_type, str), f"{name} requires a string type"
             spec_type: SpecTypeDefn
             if defn_type == DefnTypeName.s_alias:
                 spec_type = SpecTypeDefnAlias(self, name)
@@ -1346,7 +1373,7 @@ class SpecBuilder:
     def __init__(
         self,
         *,
-        api_endpoints: dict[EndpointKey, str],
+        api_endpoints: dict[EndpointKey, APIEndpointInfo],
         top_namespace: str,
         cross_output_paths: CrossOutputPaths | None,
     ) -> None:
@@ -1377,9 +1404,13 @@ class SpecBuilder:
         self.emit_id_source_enums: set[SpecTypeDefnStringEnum] = set()
 
         this_dir = os.path.dirname(os.path.realpath(__file__))
-        with open(f"{this_dir}/parts/base.py.prepart") as py_base_part:
+        with open(
+            f"{this_dir}/parts/base.py.prepart", encoding="utf-8"
+        ) as py_base_part:
             self.preparts["python"][base_namespace_name] = py_base_part.read()
-        with open(f"{this_dir}/parts/base.ts.prepart") as ts_base_part:
+        with open(
+            f"{this_dir}/parts/base.ts.prepart", encoding="utf-8"
+        ) as ts_base_part:
             self.preparts["typescript"][base_namespace_name] = ts_base_part.read()
 
         base_namespace.types["ObjectId"] = SpecTypeDefnObject(
@@ -1506,8 +1537,10 @@ class SpecBuilder:
         if len(path) == 2:
             if isinstance(defn_type, SpecTypeDefnStringEnum):
                 assert path[1].parameters is None
+                statement = f"$import: [{defn_type.namespace.name}]"
                 self.ensure(
-                    path[1].name in defn_type.values, f"missing-enum-value: {path}"
+                    path[1].name in defn_type.values,
+                    f"missing-enum-value: {path} have you specified the dependency in an import statement: {statement}",
                 )
                 return SpecTypeLiteralWrapper(
                     value=path[1].name,
