@@ -261,6 +261,10 @@ def _emit_endpoint_parameters(
     } | _emit_endpoint_parameter_examples(examples)
 
 
+def _emit_endpoint_deprecated(deprecated: bool) -> DictApiSchema:
+    return {"deprecated": True} if deprecated else {}
+
+
 def _emit_stability_level(
     stability_level: EmitOpenAPIStabilityLevel | None,
 ) -> DictApiSchema:
@@ -275,7 +279,12 @@ def _emit_stability_level(
         case EmitOpenAPIStabilityLevel.draft:
             stability_info["x-beta"] = True
         case EmitOpenAPIStabilityLevel.beta:
-            stability_info["x-badges"] = [{"name": "Beta", "color": "DarkOrange"}]
+            stability_info["x-badges"] = [
+                {
+                    "name": "Beta",
+                    "color": "DarkOrange",
+                }
+            ]
         case EmitOpenAPIStabilityLevel.stable:
             pass
         case _:
@@ -330,18 +339,57 @@ def _emit_endpoint_response_examples(
     return {"examples": response_examples}
 
 
+def _create_warning_banner(api_type: str, message: str) -> str:
+    return (
+        f'<div style="background-color: #fff3cd; border: 1px solid #ffeaa7; '
+        f'border-radius: 4px; padding: 12px; margin-bottom: 16px;">'
+        f"<strong>⚠️ {api_type} API:</strong> {message}"
+        f"</div>"
+    )
+
+
+def _get_stability_warning(
+    stability_level: EmitOpenAPIStabilityLevel | None,
+) -> str:
+    resolved_stability_level = (
+        stability_level
+        if stability_level is not None
+        else EmitOpenAPIStabilityLevel.stable
+    )
+
+    match resolved_stability_level:
+        case EmitOpenAPIStabilityLevel.draft:
+            return _create_warning_banner(
+                "Draft",
+                "This endpoint is in draft status and may change significantly. Not recommended for production use.",
+            )
+        case EmitOpenAPIStabilityLevel.beta:
+            return _create_warning_banner(
+                "Beta",
+                "This endpoint is in beta and its required parameters may change. Use with caution in production environments.",
+            )
+        case EmitOpenAPIStabilityLevel.stable:
+            return ""
+
+
 def _emit_endpoint_description(
-    description: str, guides: list[EmitOpenAPIGuide]
+    description: str,
+    guides: list[EmitOpenAPIGuide],
+    stability_level: EmitOpenAPIStabilityLevel | None = None,
 ) -> dict[str, str]:
+    stability_warning = _get_stability_warning(stability_level)
+
     full_guides = "<br/>".join([
         _write_guide_as_html(guide, is_open=False)
         for guide in sorted(guides, key=lambda g: g.ref_name)
     ])
-    return {
-        "description": description
-        if len(guides) == 0
-        else f"{description}<br/>{full_guides}"
-    }
+
+    full_description_parts = [
+        part for part in [stability_warning, description, full_guides] if part
+    ]
+    full_description = "<br/>".join(full_description_parts)
+
+    return {"description": full_description}
 
 
 def _emit_namespace(
@@ -376,7 +424,10 @@ def _emit_namespace(
                 "tags": endpoint.tags,
                 "summary": endpoint.summary,
             }
-            | _emit_endpoint_description(endpoint.description, ctx.endpoint.guides)
+            | _emit_endpoint_deprecated(endpoint.deprecated)
+            | _emit_endpoint_description(
+                endpoint.description, ctx.endpoint.guides, endpoint.stability_level
+            )
             | _emit_stability_level(endpoint.stability_level)
             | _emit_endpoint_parameters(endpoint, argument_type, ctx.endpoint.examples)
             | _emit_endpoint_request_body(
@@ -474,8 +525,18 @@ def _emit_type(
         return
 
     if isinstance(stype, builder.SpecTypeDefnUnion):
-        ctx.types[stype.name] = open_api_type(
-            ctx, stype.get_backing_type(), config=config
+        converted_discriminator_map: dict[str, OpenAPIRefType] = dict()
+        if stype.discriminator_map is not None:
+            for discriminator_value, base_type in stype.discriminator_map.items():
+                converted_base_type = open_api_type(ctx, base_type, config=config)
+                assert isinstance(converted_base_type, OpenAPIRefType)
+                converted_discriminator_map[discriminator_value] = converted_base_type
+        ctx.types[stype.name] = OpenAPIUnionType(
+            [open_api_type(ctx, p, config=config) for p in stype.types],
+            discriminator=stype.discriminator,
+            discriminator_map=converted_discriminator_map
+            if stype.discriminator_map is not None
+            else None,
         )
         return
 
@@ -549,18 +610,6 @@ def _emit_type(
     ctx.types[stype.name] = final_type
 
 
-def _emit_constant(ctx: EmitOpenAPIContext, sconst: builder.SpecConstant) -> None:
-    if sconst.value_type.is_base_type(builder.BaseTypeName.s_string):
-        value = util.encode_common_string(cast(str, sconst.value))
-    elif sconst.value_type.is_base_type(builder.BaseTypeName.s_integer):
-        value = str(sconst.value)
-    else:
-        raise Exception("invalid constant type", sconst.name)
-
-    const_name = sconst.name.upper()
-    print("_emit_constant", value, const_name)
-
-
 def _emit_endpoint(
     gctx: EmitOpenAPIGlobalContext,
     ctx: EmitOpenAPIContext,
@@ -617,6 +666,7 @@ def _emit_endpoint(
         tags=[tag_name],
         summary=f"{'/'.join(namespace.path[path_cutoff:])}",
         description=description,
+        deprecated=namespace.endpoint.deprecated,
         stability_level=namespace.endpoint.stability_level,
         examples=[
             EmitOpenAPIEndpointExample(
