@@ -1,17 +1,18 @@
 import io
 import os
+from typing import assert_never
 
 from . import builder, util
-from .builder import EndpointKey, EndpointSpecificPath
+from .builder import EndpointKey, EndpointSpecificPath, PathMapping
 from .config import TypeScriptConfig
 from .cross_output_links import get_path_links
 from .emit_io_ts import emit_type_io_ts
 from .emit_typescript_util import (
     MODIFY_NOTICE,
     EmitTypescriptContext,
+    emit_constant_ts,
     emit_namespace_imports_ts,
     emit_type_ts,
-    emit_value_ts,
     resolve_namespace_name,
     resolve_namespace_ref,
     ts_type_name,
@@ -37,6 +38,7 @@ def _emit_types(builder: builder.SpecBuilder, config: TypeScriptConfig) -> None:
             out=io.StringIO(),
             namespace=namespace,
             cross_output_paths=builder.cross_output_paths,
+            api_endpoints=builder.api_endpoints,
         )
 
         _emit_namespace(ctx, config, namespace)
@@ -78,6 +80,7 @@ def _emit_types(builder: builder.SpecBuilder, config: TypeScriptConfig) -> None:
             full.write("\n")
             full.write(MODIFY_NOTICE)
             full.write(f"// === START section from {namespace.name}.ts.part ===\n")
+            full.write("\n")
             full.write(part)
             full.write(f"// === END section from {namespace.name}.ts.part ===\n")
 
@@ -112,7 +115,7 @@ def _emit_namespace(
             emit_type_ts(ctx, stype)
 
     for sconst in namespace.constants.values():
-        _emit_constant(ctx, sconst)
+        emit_constant_ts(ctx, sconst)
 
     if namespace.endpoint is not None:
         _emit_endpoint(ctx, config, namespace, namespace.endpoint)
@@ -145,7 +148,10 @@ def _emit_endpoint(
         assert endpoint.result_type == builder.ResultType.json
 
     paths_string = get_path_links(
-        ctx.cross_output_paths, namespace, current_path_type="TypeScript"
+        ctx.cross_output_paths,
+        namespace,
+        current_path_type="TypeScript",
+        endpoint=endpoint,
     )
 
     data_loader_head = ""
@@ -155,7 +161,7 @@ def _emit_endpoint(
         assert has_data
 
         data_loader_head = (
-            'import { buildApiDataLoader, argsKey } from "unc_base/data_manager"\n'
+            'import { argsKey, buildApiDataLoader } from "unc_base/data_manager"\n'
         )
         data_loader_body = (
             "\nexport const data = buildApiDataLoader(argsKey(), apiCall)\n"
@@ -175,38 +181,56 @@ def _emit_endpoint(
     unc_base_api_imports = (
         f"appSpecificApiPath, {wrap_name}" if has_multiple_endpoints else wrap_name
     )
+    path_mapping = ctx.api_endpoints[endpoint.default_endpoint_key].path_mapping
+
+    match path_mapping:
+        case PathMapping.NO_MAPPING:
+            path_mapping_part = (
+                "\n  { pathMapping: ApplicationT.APIPathMapping.noMapping },"
+            )
+        case PathMapping.DEFAULT_MAPPING:
+            path_mapping_part = ""
+        case _:
+            assert_never(path_mapping)
+
     unc_types_imports = (
-        'import { ApplicationT } from "unc_types"\n' if has_multiple_endpoints else ""
+        'import { ApplicationT } from "unc_types"\n'
+        if has_multiple_endpoints or path_mapping_part != ""
+        else ""
     )
 
     type_path = f"unc_types/{'/'.join(namespace.path)}"
 
     if is_binary:
-        tsx_response_part = f"""import {{ {unc_base_api_imports} }} from "unc_base/api"
-import type {{ Arguments }} from "{type_path}"
-{unc_types_imports}
+        tsx_response_head = f"""import {{ {unc_base_api_imports} }} from "unc_base/api"
+"""
+        tsx_response_part = f"""import type {{ Arguments }} from "{type_path}"
+
 export type {{ Arguments }}
 """
     elif has_data and endpoint.has_attachment:
-        tsx_response_part = f"""import {{ {unc_base_api_imports}, type AttachmentResponse }} from "unc_base/api"
-import type {{ Arguments, Data }} from "{type_path}"
-{unc_types_imports}
+        tsx_response_head = f"""import {{ type AttachmentResponse, {unc_base_api_imports} }} from "unc_base/api"
+"""
+        tsx_response_part = f"""import type {{ Arguments, Data }} from "{type_path}"
+
 export type {{ Arguments, Data }}
 export type Response = AttachmentResponse<Data>
 """
     elif has_data:
-        tsx_response_part = f"""import {{ {unc_base_api_imports}, type JsonResponse }} from "unc_base/api"
-import type {{ Arguments, Data }} from "{type_path}"
-{unc_types_imports}
+        tsx_response_head = f"""import {{ {unc_base_api_imports}, type JsonResponse }} from "unc_base/api"
+"""
+        tsx_response_part = f"""import type {{ Arguments, Data }} from "{type_path}"
+
 export type {{ Arguments, Data }}
 export type Response = JsonResponse<Data>
 """
 
     else:
         assert has_deprecated_result
-        tsx_response_part = f"""import {{ {unc_base_api_imports} }} from "unc_base/api"
-import type {{ Arguments, DeprecatedResult }} from "{type_path}"
-{unc_types_imports}
+        tsx_response_head = f"""import {{ {unc_base_api_imports} }} from "unc_base/api"
+"""
+        tsx_response_part = f"""import type {{ Arguments, DeprecatedResult }} from "{type_path}"
+
 export type {{ Arguments }}
 export type Response = DeprecatedResult
 """
@@ -247,41 +271,33 @@ export type Response = DeprecatedResult
 
     # tsx_api = f"""{MODIFY_NOTICE}
     tsx_api = f"""{MODIFY_NOTICE}{paths_string}
-{data_loader_head}{tsx_response_part}
+{tsx_response_head}{data_loader_head}{unc_types_imports}{tsx_response_part}
 export const apiCall = {wrap_call}(
-  {endpoint_path_part}
+  {endpoint_path_part}{path_mapping_part}
 )
 {data_loader_body}"""
 
-    output = f"{config.routes_output}/{'/'.join(namespace.path)}.tsx"
+    output = f"{config.endpoint_to_routes_output[endpoint.default_endpoint_key]}/{'/'.join(namespace.path)}.tsx"
     util.rewrite_file(output, tsx_api)
 
     # Hacky index support, until enough is migrated to regen entirely
     # Emits the import into the UI API index file
-    index_path = f"{config.routes_output}/{'/'.join(namespace.path[0:-1])}/index.tsx"
+    index_path = f"{config.endpoint_to_routes_output[endpoint.default_endpoint_key]}/{'/'.join(namespace.path[0:-1])}/index.tsx"
     api_name = f"Api{ts_type_name(namespace.path[0 - 1])}"
     if os.path.exists(index_path):
-        with open(index_path) as index:
+        with open(index_path, encoding="utf-8") as index:
             index_data = index.read()
             need_index = index_data.find(api_name) == -1
     else:
         need_index = True
 
     if need_index:
-        with open(index_path, "a") as index:
+        with open(index_path, "a", encoding="utf-8") as index:
             print(f"Updated API Index {index_path}")
             index.write(f'import * as {api_name} from "./{namespace.path[-1]}"\n\n')
             index.write(f"export {{ {api_name} }}\n")
 
 
-def _emit_constant(ctx: EmitTypescriptContext, sconst: builder.SpecConstant) -> None:
-    ctx.out.write("\n\n")
-    ctx.out.write(MODIFY_NOTICE)
-    value = emit_value_ts(ctx, sconst.value_type, sconst.value)
-    const_name = sconst.name.upper()
-    ctx.out.write(f"export const {const_name} = {value}\n")
-
-
 def _emit_id_source(builder: builder.SpecBuilder, config: TypeScriptConfig) -> None:
     id_source_output = config.id_source_output
     if id_source_output is None:
