@@ -41,10 +41,23 @@ def type_path_of(stype: builder.SpecType) -> object:  # NamePath
             parts: list[object] = ["$literal"]
             for parameter in stype.parameters:
                 assert isinstance(parameter, builder.SpecTypeLiteralWrapper)
+                emit_value = parameter.value
+                if isinstance(parameter.value_type, builder.SpecTypeDefnObject):
+                    emit_value = parameter.value
+                    assert isinstance(emit_value, (str, bool)), (
+                        f"invalid-literal-value:{emit_value}"
+                    )
+                elif isinstance(parameter.value_type, builder.SpecTypeDefnStringEnum):
+                    key = parameter.value
+                    assert isinstance(key, str)
+                    emit_value = parameter.value_type.values[key].value
+                else:
+                    raise Exception("unhandled-literal-type")
+
                 # This allows expansion to enum literal values later
                 parts.append([
                     "$value",
-                    parameter.value,
+                    emit_value,
                     type_path_of(parameter.value_type),
                 ])
             return parts
@@ -158,9 +171,16 @@ class MapTypeAlias(MapTypeBase):
     discriminator: str | None
 
 
+@dataclasses.dataclass
+class StringEnumValue:
+    value: str
+    label: str
+    deprecated: bool = False
+
+
 @dataclasses.dataclass
 class MapStringEnum(MapTypeBase):
-    values: dict[str, str]
+    values: dict[str, StringEnumValue]
 
 
 MapType = MapTypeObject | MapTypeAlias | MapStringEnum
@@ -267,7 +287,9 @@ def _extract_and_validate_layout(
     for group in ext_info.layout.groups:
         fields = set(group.fields or [])
         for field in fields:
-            assert field in stype.properties, f"layout-refers-to-missing-field:{field}"
+            assert field in stype.properties or field == DISCRIMINATOR_COMMON_NAME, (
+                f"layout-refers-to-missing-field:{field}"
+            )
 
         local_ref_name = None
         if group.ref_name is not None:
@@ -314,6 +336,9 @@ def _pull_property_from_type_recursively(
     return _pull_property_from_type_recursively(stype.base, property_name)
 
 
+DISCRIMINATOR_COMMON_NAME = "type"
+
+
 def _validate_type_ext_info(
     stype: builder.SpecTypeDefnObject,
 ) -> tuple[ExtInfoLayout | None, type_info_t.ExtInfo | None]:
@@ -324,12 +349,16 @@ def _validate_type_ext_info(
     if ext_info.label_fields is not None:
         assert stype.properties is not None
         for name in ext_info.label_fields:
+            if name == DISCRIMINATOR_COMMON_NAME:
+                continue
             prop = _pull_property_from_type_recursively(stype, name)
             assert prop is not None, f"missing-label-field:{name}"
 
     if ext_info.actions is not None:
         assert stype.properties is not None
         for action in ext_info.actions:
+            if action.property == DISCRIMINATOR_COMMON_NAME:
+                continue
             prop = _pull_property_from_type_recursively(stype, action.property)
             assert prop is not None, f"missing-action-field:{action.property}"
 
@@ -414,7 +443,11 @@ def _build_map_type(
             # IMPROVE: We probably want the label here, but this requires a change
             # to the front-end type-info and form code to handle
             values={
-                entry.value: (entry.label or entry.name)
+                entry.value: StringEnumValue(
+                    value=entry.value,
+                    label=entry.label or entry.name,
+                    deprecated=entry.deprecated,
+                )
                 for entry in stype.values.values()
             },
         )
