@@ -75,9 +75,14 @@ def _emit_function_wrapper(function: value_spec_t.Function) -> str:
             ):
                 python_type += " | None"
                 any_pass_null = True
+
+            if python_type.startswith("base_t.ExtJsonValue"):
+                return_statement = f"self._extract({index})"
+            else:
+                return_statement = f"cast({python_type}, self._extract({index}))"
             out.write(
                 f"""{INDENT}def get_{argument.ref_name}(self) -> {python_type}:
-{INDENT}{INDENT}return cast({python_type}, self._extract({index}))
+{INDENT}{INDENT}return {return_statement}
 """
             )
         out.write("\n")
