@@ -18,12 +18,12 @@ def get_server_env() -> str | None:
     return os.environ.get("UNC_SERVER_ENV")
 
 
-def get_webhook_server_port() -> int:
-    return int(os.environ.get("UNC_WEBHOOK_SERVER_PORT", 5001))
+def get_http_server_port() -> int:
+    return int(os.environ.get("UNC_WEBHOOK_SERVER_PORT", "5001"))
 
 
 def get_local_admin_server_port() -> int:
-    return int(os.environ.get("UNC_ADMIN_SERVER_PORT", 50051))
+    return int(os.environ.get("UNC_ADMIN_SERVER_PORT", "50051"))
 
 
 def get_otel_enabled() -> bool:
