@@ -41,7 +41,7 @@ def _get_extension(filename: str) -> str | None:
 
 def _run_keyword_detection(data: io.BytesIO, keyword: str) -> bool:
     try:
-        text = io.TextIOWrapper(data)
+        text = io.TextIOWrapper(data, encoding="utf-8")
         for line in text:
             if (
                 keyword in line
