@@ -4,6 +4,8 @@ from .types import (
     CommandEnqueueJob,
     CommandEnqueueJobResponse,
     CommandQueue,
+    CommandRetryJob,
+    CommandRetryJobResponse,
     CommandServerBadResponse,
     CommandServerException,
     CommandServerTimeout,
@@ -16,6 +18,8 @@ __all__: list[str] = [
     "send_job_queue_message",
     "CommandEnqueueJob",
     "CommandEnqueueJobResponse",
+    "CommandRetryJob",
+    "CommandRetryJobResponse",
     "CommandTask",
     "CommandQueue",
     "CommandServerTimeout",
