@@ -8,6 +8,8 @@ from uncountable.types import queued_job_t
 
 class CommandType(StrEnum):
     ENQUEUE_JOB = "enqueue_job"
+    RETRY_JOB = "retry_job"
+    VACCUUM_QUEUED_JOBS = "vaccuum_queued_jobs"
 
 
 RT = typing.TypeVar("RT")
@@ -24,6 +26,16 @@ class CommandEnqueueJobResponse:
     queued_job_uuid: str
 
 
+@dataclass(kw_only=True)
+class CommandRetryJobResponse:
+    queued_job_uuid: str | None
+
+
+@dataclass(kw_only=True)
+class CommandVaccuumQueuedJobsResponse:
+    pass
+
+
 @dataclass(kw_only=True)
 class CommandEnqueueJob(CommandBase[CommandEnqueueJobResponse]):
     type: CommandType = CommandType.ENQUEUE_JOB
@@ -32,7 +44,18 @@ class CommandEnqueueJob(CommandBase[CommandEnqueueJobResponse]):
     response_queue: asyncio.Queue[CommandEnqueueJobResponse]
 
 
-_Command = CommandEnqueueJob
+@dataclass(kw_only=True)
+class CommandRetryJob(CommandBase[CommandRetryJobResponse]):
+    type: CommandType = CommandType.RETRY_JOB
+    queued_job_uuid: str
+
+
+@dataclass(kw_only=True)
+class CommandVaccuumQueuedJobs(CommandBase[CommandVaccuumQueuedJobsResponse]):
+    type: CommandType = CommandType.VACCUUM_QUEUED_JOBS
+
+
+_Command = CommandEnqueueJob | CommandRetryJob | CommandVaccuumQueuedJobs
 
 
 CommandQueue = asyncio.Queue[_Command]
