@@ -1,7 +1,9 @@
-from sqlalchemy import JSON, BigInteger, Column, DateTime, Text
+from sqlalchemy import JSON, BigInteger, Column, DateTime, Enum, Text
 from sqlalchemy.orm import declarative_base
 from sqlalchemy.sql import func
 
+from uncountable.types import queued_job_t
+
 Base = declarative_base()
 
 
@@ -15,3 +17,8 @@ class QueuedJob(Base):
     )
     payload = Column(JSON, nullable=False)
     num_attempts = Column(BigInteger, nullable=False, default=0, server_default="0")
+    status = Column(
+        Enum(queued_job_t.JobStatus, length=None),
+        default=queued_job_t.JobStatus.QUEUED,
+        nullable=True,
+    )
