@@ -6,14 +6,17 @@ from dataclasses import dataclass
 
 from opentelemetry.trace import get_current_span
 
-from uncountable.integration.db.connect import IntegrationDBService, create_db_engine
-from uncountable.integration.db.session import get_session_maker
 from uncountable.integration.queue_runner.command_server import (
     CommandEnqueueJob,
     CommandEnqueueJobResponse,
     CommandQueue,
+    CommandRetryJob,
+    CommandRetryJobResponse,
     CommandTask,
 )
+from uncountable.integration.queue_runner.command_server.types import (
+    CommandVaccuumQueuedJobs,
+)
 from uncountable.integration.queue_runner.datastore import DatastoreSqlite
 from uncountable.integration.queue_runner.datastore.interface import Datastore
 from uncountable.integration.queue_runner.worker import Worker
@@ -83,14 +86,11 @@ def _start_workers(
     return job_worker_lookup
 
 
-async def start_scheduler(command_queue: CommandQueue) -> None:
+async def start_scheduler(
+    command_queue: CommandQueue, datastore: DatastoreSqlite
+) -> None:
     logger = Logger(get_current_span())
     result_queue: ResultQueue = asyncio.Queue()
-    engine = create_db_engine(IntegrationDBService.RUNNER)
-    session_maker = get_session_maker(engine)
-
-    datastore = DatastoreSqlite(session_maker)
-    datastore.setup(engine)
 
     with ProcessPoolExecutor(max_workers=_MAX_JOB_WORKERS) as process_pool:
         job_worker_lookup = _start_workers(
@@ -104,7 +104,9 @@ async def start_scheduler(command_queue: CommandQueue) -> None:
                 worker = job_worker_lookup[queued_job.job_ref_name]
             except KeyError as e:
                 logger.log_exception(e)
-                datastore.remove_job_from_queue(queued_job.queued_job_uuid)
+                datastore.update_job_status(
+                    queued_job.queued_job_uuid, queued_job_t.JobStatus.FAILED
+                )
                 return
             await worker.listen_queue.put(queued_job)
 
@@ -119,17 +121,11 @@ async def start_scheduler(command_queue: CommandQueue) -> None:
                     queued_job_t.InvocationContextManual,
                 ),
             ):
-                existing_queued_jobs = datastore.load_job_queue()
-                duplicate_job = next(
-                    (
-                        job
-                        for job in existing_queued_jobs
-                        if job.job_ref_name == job_ref_name
-                    ),
-                    None,
+                existing_queued_job = datastore.get_next_queued_job_for_ref_name(
+                    job_ref_name=job_ref_name
                 )
-                if duplicate_job is not None:
-                    return duplicate_job.queued_job_uuid
+                if existing_queued_job is not None:
+                    return existing_queued_job.queued_job_uuid
             queued_job = datastore.add_job_to_queue(
                 job_payload=payload,
                 job_ref_name=job_ref_name,
@@ -146,6 +142,25 @@ async def start_scheduler(command_queue: CommandQueue) -> None:
                 CommandEnqueueJobResponse(queued_job_uuid=queued_job_uuid)
             )
 
+        async def _handle_retry_job_command(command: CommandRetryJob) -> None:
+            queued_job = datastore.retry_job(command.queued_job_uuid)
+            if queued_job is None:
+                await command.response_queue.put(
+                    CommandRetryJobResponse(queued_job_uuid=None)
+                )
+                return
+
+            await enqueue_queued_job(queued_job)
+            await command.response_queue.put(
+                CommandRetryJobResponse(queued_job_uuid=queued_job.queued_job_uuid)
+            )
+
+        def _handle_vaccuum_queued_jobs_command(
+            command: CommandVaccuumQueuedJobs,
+        ) -> None:
+            logger.log_info("Vaccuuming queued jobs...")
+            datastore.vaccuum_queued_jobs()
+
         for queued_job in queued_jobs:
             await enqueue_queued_job(queued_job)
 
@@ -162,10 +177,24 @@ async def start_scheduler(command_queue: CommandQueue) -> None:
                     match command:
                         case CommandEnqueueJob():
                             await _handle_enqueue_job_command(command=command)
+                        case CommandRetryJob():
+                            await _handle_retry_job_command(command=command)
+                        case CommandVaccuumQueuedJobs():
+                            _handle_vaccuum_queued_jobs_command(command=command)
                         case _:
                             typing.assert_never(command)
                     command_task = asyncio.create_task(command_queue.get())
                 elif task == result_task:
                     queued_job_result = result_task.result()
-                    datastore.remove_job_from_queue(queued_job_result.queued_job_uuid)
+                    match queued_job_result.job_result.success:
+                        case True:
+                            datastore.update_job_status(
+                                queued_job_result.queued_job_uuid,
+                                queued_job_t.JobStatus.SUCCESS,
+                            )
+                        case False:
+                            datastore.update_job_status(
+                                queued_job_result.queued_job_uuid,
+                                queued_job_t.JobStatus.FAILED,
+                            )
                     result_task = asyncio.create_task(result_queue.get())
